package cn.com.ctop.job.bytedance.data.entity;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.Data;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Data
public class ByteDanceV3AdvertiserReportDTO {

    private static final long serialVersionUID = 3674054148513217865L;

    private Long advertiserId;
    private String dimensions;
    private String metrics;
    private String startTime;
    private String endTime;
    private Integer pageSize;
    private Integer page;
    private String orderBy;
    private String filters;
    private String dataTopic;

    public ByteDanceV3AdvertiserReportDTO() {
    }

    private static final ObjectMapper mapper = new ObjectMapper();

    public ByteDanceV3AdvertiserReportDTO(Long advertiserId, List<String> dimensions, String dataTopic, String startTime, String endTime, Integer pageSize) {
        try {
            this.advertiserId = advertiserId;
            this.dimensions = mapper.writeValueAsString(dimensions);

            this.dataTopic = dataTopic;
            if ("MATERIAL_DATA".equals(dataTopic)) {
                this.metrics = mapper.writeValueAsString(getParamsMaterialFields());
            }else{
                this.metrics = mapper.writeValueAsString(getParamsFields());
            }

            this.startTime = startTime;
            this.endTime = endTime;
            this.pageSize = pageSize;
            List<JSONObject> orderBy = new ArrayList<>();
            JSONObject ord = new JSONObject();
            ord.put("field", "stat_cost");
            ord.put("type", "ASC");
            orderBy.add(ord);
            this.orderBy = mapper.writeValueAsString(orderBy);

            List<JSONObject> filters = new ArrayList<>();
            JSONObject filter = new JSONObject();
            filter.put("field", "stat_cost");
            filter.put("type", 3);
            filter.put("operator", 4);
            filter.put("values", Arrays.asList("0"));
            filters.add(filter);
            this.filters = mapper.writeValueAsString(filters);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
    }

    public static List<String> getParamsFields() {
        List<String> fields = new ArrayList<>();
        fields.add("stat_cost");
        fields.add("show_cnt");
        fields.add("cpm_platform");
        fields.add("click_cnt");
        fields.add("ctr");
        fields.add("cpc_platform");
        fields.add("attribution_convert_cnt");
        fields.add("attribution_convert_cost");
        fields.add("attribution_conversion_rate");
        fields.add("attribution_deep_convert_cnt");
        fields.add("attribution_deep_convert_cost");
        fields.add("attribution_deep_convert_rate");
        fields.add("convert_cnt");
        fields.add("conversion_cost");
        fields.add("conversion_rate");
        fields.add("deep_convert_cnt");
        fields.add("deep_convert_cost");
        fields.add("deep_convert_rate");
        fields.add("click_start_cnt");
        fields.add("click_start_cost");
        fields.add("click_start_rate");
        fields.add("download_finish_cnt");
        fields.add("download_finish_cost");
        fields.add("download_finish_rate");
        fields.add("install_finish_cnt");
        fields.add("install_finish_cost");
        fields.add("install_finish_rate");
        fields.add("active");
        fields.add("active_cost");
        fields.add("active_rate");
        fields.add("active_register");
        fields.add("active_register_cost");
        fields.add("active_register_rate");
        fields.add("game_addiction");
        fields.add("game_addiction_cost");
        fields.add("game_addiction_rate");
        fields.add("attribution_next_day_open_cnt");
        fields.add("attribution_next_day_open_cost");
        fields.add("attribution_next_day_open_rate");
        fields.add("next_day_open");
        fields.add("active_pay");
        fields.add("active_pay_cost");
        fields.add("active_pay_rate");
        fields.add("game_pay_count");
        fields.add("game_pay_cost");
        fields.add("attribution_game_pay_7d_count");
        fields.add("attribution_game_pay_7d_cost");
        fields.add("attribution_active_pay_7d_per_count");
        fields.add("in_app_uv");
        fields.add("in_app_detail_uv");
        fields.add("in_app_cart");
        fields.add("in_app_pay");
        fields.add("in_app_order");
        fields.add("attribution_retention_2d_cnt");
        fields.add("attribution_retention_2d_cost");
        fields.add("attribution_retention_2d_rate");
        fields.add("attribution_retention_3d_cnt");
        fields.add("attribution_retention_3d_cost");
        fields.add("attribution_retention_3d_rate");
        fields.add("attribution_retention_4d_cnt");
        fields.add("attribution_retention_4d_cost");
        fields.add("attribution_retention_4d_rate");
        fields.add("attribution_retention_5d_cnt");
        fields.add("attribution_retention_5d_cost");
        fields.add("attribution_retention_5d_rate");
        fields.add("attribution_retention_6d_cnt");
        fields.add("attribution_retention_6d_cost");
        fields.add("attribution_retention_6d_rate");
        fields.add("attribution_retention_7d_cnt");
        fields.add("attribution_retention_7d_cost");
        fields.add("attribution_retention_7d_rate");
        fields.add("attribution_retention_7d_sum_cnt");
        fields.add("attribution_retention_7d_total_cost");
        fields.add("attribution_billing_game_pay_7d_count");
        fields.add("attribution_billing_game_pay_7d_cost");
        fields.add("attribution_billing_game_in_app_ltv_1day");
        fields.add("attribution_billing_game_in_app_ltv_2days");
        fields.add("attribution_billing_game_in_app_ltv_3days");
        fields.add("attribution_billing_game_in_app_ltv_4days");
        fields.add("attribution_billing_game_in_app_ltv_5days");
        fields.add("attribution_billing_game_in_app_ltv_6days");
        fields.add("attribution_billing_game_in_app_ltv_7days");
        fields.add("attribution_billing_game_in_app_roi_1day");
        fields.add("attribution_billing_game_in_app_roi_2days");
        fields.add("attribution_billing_game_in_app_roi_3days");
        fields.add("attribution_billing_game_in_app_roi_4days");
        fields.add("attribution_billing_game_in_app_roi_5days");
        fields.add("attribution_billing_game_in_app_roi_6days");
        fields.add("attribution_billing_game_in_app_roi_7days");
        fields.add("attribution_active_pay");
        fields.add("attribution_active_pay_cost");
        fields.add("attribution_active_pay_rate");
        fields.add("stat_pay_amount");
        fields.add("pay_amount_roi");
        fields.add("phone");
        fields.add("form");
        fields.add("form_submit");
        fields.add("map");
        fields.add("button");
        fields.add("view");
        fields.add("download_start");
        fields.add("qq");
        fields.add("lottery");
        fields.add("vote");
        fields.add("message");
        fields.add("redirect");
        fields.add("shopping");
        fields.add("consult");
        fields.add("consult_effective");
        fields.add("phone_confirm");
        fields.add("phone_connect");
        fields.add("phone_effective");
        fields.add("coupon");
        fields.add("coupon_single_page");
        fields.add("redirect_to_shop");
        fields.add("poi_address_click");
        fields.add("poi_collect");
        fields.add("customer_effective");
        fields.add("attribution_customer_effective");
        fields.add("attribution_customer_effective_cost");
        fields.add("attribution_clue_pay_succeed");
        fields.add("attribution_clue_pay_succeed_cost");
        fields.add("attribution_clue_interflow");
        fields.add("attribution_clue_interflow_cost");
        fields.add("attribution_clue_high_intention");
        fields.add("attribution_clue_high_intention_cost");
        fields.add("attribution_clue_confirm");
        fields.add("attribution_clue_confirm_cost");
        fields.add("consult_clue");
        fields.add("attribution_work_wechat_added_count");
        fields.add("attribution_work_wechat_unfriend_count");
        fields.add("attribution_game_in_app_ltv_1day");
        fields.add("attribution_game_in_app_ltv_2days");
        fields.add("attribution_game_in_app_ltv_3days");
        fields.add("attribution_game_in_app_ltv_4days");
        fields.add("attribution_game_in_app_ltv_5days");
        fields.add("attribution_game_in_app_ltv_6days");
        fields.add("attribution_game_in_app_ltv_7days");
        fields.add("attribution_game_in_app_ltv_8days");
        fields.add("attribution_game_in_app_roi_1day");
        fields.add("attribution_game_in_app_roi_2days");
        fields.add("attribution_game_in_app_roi_3days");
        fields.add("attribution_game_in_app_roi_4days");
        fields.add("attribution_game_in_app_roi_5days");
        fields.add("attribution_game_in_app_roi_6days");
        fields.add("attribution_game_in_app_roi_7days");
        fields.add("attribution_game_in_app_roi_8days");
        fields.add("attribution_day_active_pay_count");
        fields.add("attribution_day_active_pay_cost");
        fields.add("attribution_day_active_pay_rate");
        fields.add("active_pay_intra_day_count");
        fields.add("active_pay_intra_day_cost");
        fields.add("active_pay_intra_day_rate");
        fields.add("attribution_micro_game_0d_ltv");
        fields.add("attribution_micro_game_3d_ltv");
        fields.add("attribution_micro_game_7d_ltv");
        fields.add("attribution_micro_game_0d_roi");
        fields.add("attribution_micro_game_3d_roi");
        fields.add("attribution_micro_game_7d_roi");
        fields.add("loan_completion");
        fields.add("loan_completion_cost");
        fields.add("loan_completion_rate");
        fields.add("pre_loan_credit");
        fields.add("pre_loan_credit_cost");
        fields.add("loan_credit");
        fields.add("loan_credit_cost");
        fields.add("loan_credit_rate");
        fields.add("loan");
        fields.add("loan_cost");
        fields.add("loan_rate");
        fields.add("premium_payment_count");
        fields.add("premium_payment_cost");
        fields.add("insurance_lt_roi");
        fields.add("bankcard_information_count");
        fields.add("personal_information_count");
        fields.add("certification_information_count");
        fields.add("open_account_count");
        fields.add("first_class_count");
        fields.add("second_class_count");
        fields.add("unfollow_in_wechat_count");
        fields.add("in_wechat_pay_count");
        fields.add("attribution_work_wechat_dialog_count");
        fields.add("low_loan_credit_count");
        fields.add("high_loan_credit_count");
        fields.add("withdraw_m2_count");
        fields.add("attribution_conversion_class_count");
        fields.add("in_app_order_gmv");
        fields.add("in_app_order_roi");
        fields.add("in_app_pay_gmv");
        fields.add("in_app_pay_roi");
        fields.add("total_play");
        fields.add("play_duration_3s");
        fields.add("valid_play");
        fields.add("valid_play_cost");
        fields.add("valid_play_rate");
        fields.add("valid_play_of_mille");
        fields.add("valid_play_cost_of_mille");
        fields.add("play_25_feed_break");
        fields.add("play_50_feed_break");
        fields.add("play_75_feed_break");
        fields.add("play_99_feed_break");
        fields.add("average_play_time_per_play");
        fields.add("play_over_rate");
        fields.add("wifi_play_rate");
        fields.add("card_show");
        fields.add("dy_like");
        fields.add("dy_comment");
        fields.add("dy_share");
        fields.add("ad_dislike_cnt");
        fields.add("ad_report_cnt");
        fields.add("ies_challenge_click");
        fields.add("ies_music_click");
        fields.add("location_click");
        fields.add("dy_home_visited");
        fields.add("dy_follow");
        fields.add("message_action");
        fields.add("click_landing_page");
        fields.add("click_shopwindow");
        fields.add("click_website");
        fields.add("click_call_dy");
        fields.add("click_download");
        fields.add("luban_live_enter_cnt");
        fields.add("live_watch_one_minute_count");
        fields.add("luban_live_follow_cnt");
        fields.add("luban_live_share_cnt");
        fields.add("luban_live_comment_cnt");
        fields.add("live_component_click_count");
        fields.add("live_component_click_cost");
        fields.add("live_component_click_rate");
        return fields;
    }


    public static List<String> getParamsMaterialFields() {
        List<String> fields = new ArrayList<>();
        fields.add("stat_cost");
        fields.add("cpm_platform");
        fields.add("show_cnt");
        fields.add("click_cnt");
        fields.add("ctr");
        fields.add("cpc_platform");
        fields.add("convert_cnt");
        fields.add("conversion_cost");
        fields.add("conversion_rate");
        fields.add("deep_convert_cnt");
        fields.add("deep_convert_cost");
        fields.add("deep_convert_rate");
        fields.add("click_start_cnt");
        fields.add("click_start_cost");
        fields.add("click_start_rate");
        fields.add("download_finish_cnt");
        fields.add("download_finish_cost");
        fields.add("download_finish_rate");
        fields.add("install_finish_cnt");
        fields.add("install_finish_cost");
        fields.add("install_finish_rate");
        fields.add("active");
        fields.add("active_cost");
        fields.add("active_rate");
        fields.add("active_register");
        fields.add("active_register_cost");
        fields.add("active_register_rate");
        fields.add("game_addiction");
        fields.add("game_addiction_cost");
        fields.add("game_addiction_rate");
        fields.add("attribution_next_day_open_cnt");
        fields.add("attribution_next_day_open_cost");
        fields.add("attribution_next_day_open_rate");
        fields.add("next_day_open");
        fields.add("active_pay");
        fields.add("active_pay_cost");
        fields.add("active_pay_rate");
        fields.add("game_pay_count");
        fields.add("game_pay_cost");
        fields.add("attribution_game_pay_7d_count");
        fields.add("attribution_game_pay_7d_cost");
        fields.add("in_app_uv");
        fields.add("in_app_detail_uv");
        fields.add("in_app_cart");
        fields.add("in_app_pay");
        fields.add("in_app_order");
        fields.add("attribution_game_in_app_ltv_1day");
        fields.add("attribution_game_in_app_ltv_2days");
        fields.add("attribution_game_in_app_ltv_3days");
        fields.add("attribution_game_in_app_ltv_4days");
        fields.add("attribution_game_in_app_ltv_5days");
        fields.add("attribution_game_in_app_ltv_6days");
        fields.add("attribution_game_in_app_ltv_7days");
        fields.add("attribution_game_in_app_ltv_8days");
        fields.add("attribution_game_in_app_roi_1day");
        fields.add("attribution_game_in_app_roi_2days");
        fields.add("attribution_game_in_app_roi_3days");
        fields.add("attribution_game_in_app_roi_4days");
        fields.add("attribution_game_in_app_roi_5days");
        fields.add("attribution_game_in_app_roi_6days");
        fields.add("attribution_game_in_app_roi_7days");
        fields.add("attribution_game_in_app_roi_8days");
        fields.add("active_pay_intra_day_count");
        fields.add("active_pay_intra_day_cost");
        fields.add("active_pay_intra_day_rate");
        return fields;
    }

}
