package cn.com.ctop.job.bytedance.data.service.impl;

import cn.com.ctop.job.bytedance.data.constant.BytedanceConstant;
import cn.com.ctop.job.bytedance.data.entity.BytedanceCampaignReportHourly;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.mapper.BytedanceCampaignReportHourlyMapper;
import cn.com.ctop.job.bytedance.data.service.IBytedanceCampaignReportHourlyService;
import cn.com.ctop.job.bytedance.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 获取定向包
 *
 * @author jeecg-boot
 * 2021-10-21
 * @version V1.0
 */
@Service
@Slf4j
public class BytedanceCampaignReportHourlyServiceImpl implements IBytedanceCampaignReportHourlyService {
    @Value("${api.bytedance.api-prefix}")
    private String bytedanceApiUrl;
    @Resource
    BytedanceCampaignReportHourlyMapper bytedanceCampaignReportHourlyMapper;

    @Override
    public void getCampaignHourlyReport(OauthToken oauthToken, String yesterda, String yesterda1, String bytedanceReportTypeDaily, int pageNumber, int pageSize) {
        log.info("开始获取广告组时报数据 accountID:{},page:{}",oauthToken.getAccountId(),pageNumber);
        // 请求地址
        String url = bytedanceApiUrl + BytedanceConstant.BYTEDANCE_V2_REPORT_CAMPAIGN_GET;
        ;
        JSONObject param = new JSONObject();
        param.put("advertiser_id", oauthToken.getAccountId());
        param.put("start_date", yesterda);
        param.put("end_date", yesterda1);
        param.put("time_granularity", bytedanceReportTypeDaily);
        param.put("page", pageNumber);
        param.put("page_size", pageSize);
        JSONArray groupBy = new JSONArray();
        groupBy.add("STAT_GROUP_BY_FIELD_ID");
        groupBy.add("STAT_GROUP_BY_FIELD_STAT_TIME");
        param.put("group_by", groupBy);
        JSONObject resultObject = HttpUtils.bytedanceGetRequest(oauthToken.getAccessToken(), url, param);

        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取广告组接口异常==》accountId:{},message:{}", oauthToken.getAccountId(), resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            return;
        }
        List<BytedanceCampaignReportHourly> list = new ArrayList<>();
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            //时报
            BytedanceCampaignReportHourly report = JSONObject.parseObject(dataObject.toJSONString(), BytedanceCampaignReportHourly.class);
            report.setAdvertiserId(oauthToken.getAccountId());
            String hour = report.getStatDatetime().substring(11, 13);
            report.setStatDatetime(yesterda.replace("-", ""));
            report.setHour(Integer.parseInt(hour));
            list.add(report);
        }
        bytedanceCampaignReportHourlyMapper.insert(list);
        pageNumber++;
        int totalPage = resultObject.getJSONObject("data").getJSONObject("page_info").getInteger("total_page");
        if (pageNumber <= totalPage) {
            getCampaignHourlyReport(oauthToken, yesterda, yesterda1, bytedanceReportTypeDaily, pageNumber, 1000);
        }

    }

}
