package cn.com.ctop.job.bytedance.data.service.impl;

import cn.com.ctop.job.bytedance.data.constant.BytedanceConstant;
import cn.com.ctop.job.bytedance.data.entity.BytedanceCreativeGet;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.mapper.BytedanceCreativeGetMapper;
import cn.com.ctop.job.bytedance.data.service.IBytedanceCreativeGetService;
import cn.com.ctop.job.bytedance.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 创意信息
 *
 * @author jeecg-boot
 * 2021-10-22
 * @version V1.0
 */
@Service
@Slf4j
public class BytedanceCreativeGetServiceImpl implements IBytedanceCreativeGetService {
    @Value("${api.bytedance.api-prefix}")
    private String bytedanceApiUrl;
    @Resource
    BytedanceCreativeGetMapper bytedanceCreativeGetMapper;

    @Override
    public void getCreativeData(OauthToken oauthToken, String startDate, String endDate, int page, int pageSize) {
        log.info("开始获取创意数据 accountID:{},page:{}", oauthToken.getAccountId(), page);
        // 请求地址
        String url = bytedanceApiUrl + BytedanceConstant.BYTEDANCE_V2_CREATIVE_GET;
        JSONObject param = new JSONObject();
        param.put("advertiser_id", oauthToken.getAccountId());
        param.put("page", page);
        param.put("page_size", pageSize);
        JSONObject param2 = new JSONObject();
        param2.put("creative_modify_time", startDate);
        param.put("filtering", param2);

        JSONObject resultObject = HttpUtils.bytedanceGetRequest(oauthToken.getAccessToken(), url, param);

        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取创意接口异常==》accountId:{},message:{}", oauthToken.getAccountId(), resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            return;
        }
        List<BytedanceCreativeGet> list = new ArrayList<>();
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            BytedanceCreativeGet report = JSONObject.parseObject(dataObject.toJSONString(), BytedanceCreativeGet.class);
            list.add(report);
        }
        bytedanceCreativeGetMapper.insert(list);
        page++;
        int totalPage = resultObject.getJSONObject("data").getJSONObject("page_info").getInteger("total_page");
        if (page <= totalPage) {
            getCreativeData(oauthToken, startDate, endDate, page, 1000);
        }

    }
}
