package cn.com.ctop.job.bytedance.data.service.impl;

import cn.com.ctop.job.bytedance.data.constant.BytedanceConstant;
import cn.com.ctop.job.bytedance.data.entity.BytedanceCreativeReportDaily;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.mapper.BytedanceCreativeReportDailyMapper;
import cn.com.ctop.job.bytedance.data.service.IBytedanceCreativeReportDailyService;
import cn.com.ctop.job.bytedance.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 广告创意
 *
 * @author jeecg-boot
 * 2021-10-22
 * @version V1.0
 */
@Service
@Slf4j
public class BytedanceCreativeReportDailyServiceImpl implements IBytedanceCreativeReportDailyService {
    @Value("${api.bytedance.api-prefix}")
    private String bytedanceApiUrl;
    @Resource
    BytedanceCreativeReportDailyMapper bytedanceCreativeReportDailyMapper;

    @Override
    public void getCreativeDailyReport(OauthToken oauthToken, String yesterda, String yesterda1, String bytedanceReportTypeDaily, int pageNumber, int pageSize) {
        log.info("开始获取创意日报数据 accountID:{},page:{}", oauthToken.getAccountId(), pageNumber);
        // 请求地址
        String url = bytedanceApiUrl + BytedanceConstant.BYTEDANCE_V2_REPORT_CREATIVE_GET;
        ;
        JSONObject param = new JSONObject();
        param.put("advertiser_id", oauthToken.getAccountId());
        param.put("start_date", yesterda);
        param.put("end_date", yesterda1);
        param.put("time_granularity", bytedanceReportTypeDaily);
        param.put("page", pageNumber);
        param.put("page_size", pageSize);
        JSONArray groupBy = new JSONArray();
        groupBy.add("STAT_GROUP_BY_FIELD_ID");
        groupBy.add("STAT_GROUP_BY_FIELD_STAT_TIME");
        param.put("group_by", groupBy);

        JSONObject resultObject = HttpUtils.bytedanceGetRequest(oauthToken.getAccessToken(), url, param);

        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取广告创意接口异常==》accountId:{},message:{}", oauthToken.getAccountId(), resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            return;
        }
        List<BytedanceCreativeReportDaily> list = new ArrayList<>();
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            //时报
            BytedanceCreativeReportDaily report = JSONObject.parseObject(dataObject.toJSONString(), BytedanceCreativeReportDaily.class);

            report.setAdvertiserId(oauthToken.getAccountId());
            report.setStatDatetime(report.getStatDatetime() == null ? null : report.getStatDatetime().substring(0, 10).replace("-", ""));
            list.add(report);
        }
        try {
            bytedanceCreativeReportDailyMapper.insert(list);
        }catch (Exception e){
            log.info(e.toString());
        }

        pageNumber++;
        int totalPage = resultObject.getJSONObject("data").getJSONObject("page_info").getInteger("total_page");
        if (pageNumber <= totalPage) {
            getCreativeDailyReport(oauthToken, yesterda, yesterda1, bytedanceReportTypeDaily, pageNumber, 1000);
        }
    }
}
