package cn.com.ctop.job.bytedance.data.service.impl;

import cn.com.ctop.job.bytedance.data.constant.BytedanceConstant;
import cn.com.ctop.job.bytedance.data.entity.BytedanceFileImageGet;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.mapper.BytedanceFileImageGetMapper;
import cn.com.ctop.job.bytedance.data.service.IBytedanceFileImageGetService;
import cn.com.ctop.job.bytedance.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 头条图片素材
 *
 * @author jeecg-boot
 * 2021-10-22
 * @version V1.0
 */
@Service
@Slf4j
public class BytedanceFileImageGetServiceImpl implements IBytedanceFileImageGetService {
    @Value("${api.bytedance.api-prefix}")
    private String bytedanceApiUrl;
    @Resource
    BytedanceFileImageGetMapper bytedanceFileImageGetMapper;

    @Override
    public void getFileImageData(OauthToken oauthToken, String startDate, String endDate, int page, int pageSize) {
        log.info("开始获取图片素材 accountID:{},page:{}", oauthToken.getAccountId(), page);
        // 请求地址
        String url = bytedanceApiUrl + BytedanceConstant.BYTEDANCE_V2_FILE_IMAGE_GET;
        JSONObject param = new JSONObject();
        param.put("advertiser_id", oauthToken.getAccountId());
        param.put("page", page);
        param.put("page_size", pageSize);
        JSONObject param2 = new JSONObject();
        param2.put("start_time", startDate);
        param2.put("end_time", endDate);
        param.put("filtering", param2);
        JSONObject resultObject = HttpUtils.bytedanceGetRequest(oauthToken.getAccessToken(), url, param);

        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取图片素材接口异常==》accountId:{},message:{}", oauthToken.getAccountId(), resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            return;
        }
        List<BytedanceFileImageGet> list = new ArrayList<>();
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            try {
                BytedanceFileImageGet report = JSONObject.parseObject(dataObject.toJSONString(), BytedanceFileImageGet.class);
                report.setAdvertiserId(oauthToken.getAccountId());
                if (null != report.getId() && report.getId().contains("web")) {
                    report.setImageUrl("https://sf6-ttcdn-tos.pstatp.com/obj/" + report.getId());
                } else {
                    report.setImageUrl("https://sf6-ttcdn-tos.pstatp.com/obj/mosaic-legacy/" + report.getId());
                }
                list.add(report);
            } catch (Exception e) {
                log.info("错误:{}", e.toString());
            }
        }
        bytedanceFileImageGetMapper.insert(list);
        page++;
        int totalPage = resultObject.getJSONObject("data").getJSONObject("page_info").getInteger("total_page");
        if (page <= totalPage) {
            getFileImageData(oauthToken, startDate, endDate, page, 100);
        }
    }
}
