package cn.com.ctop.job.bytedance.data.service.impl;

import cn.com.ctop.job.bytedance.data.constant.BytedanceConstant;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.mapper.BytedanceFundMapper;
import cn.com.ctop.job.bytedance.data.service.IBytedanceFundService;
import cn.com.ctop.job.bytedance.data.utils.Check;
import cn.com.ctop.job.bytedance.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

@Service
@Slf4j
public class BytedanceFundServiceImpl implements IBytedanceFundService {
    @Autowired
    private BytedanceFundMapper bytedanceFundMapper;


    @Override
    public void getBytedanceFundInfo(OauthToken oauthToken) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", oauthToken.getAccessToken());
        JSONObject params = new JSONObject();
        params.put("advertiser_id", oauthToken.getAccountId());
        params.put("grant_type_split", "ON");
        String resultStr = HttpUtils.httpGetRequest(BytedanceConstant.BYTEDANCE_FUND_GET, headers, params);
        if (!Check.isNull(resultStr)) {
            JSONObject result = JSONObject.parseObject(resultStr);
            if (!Check.isNull(result)) {
                Integer code = result.getInteger("code");
                if (code != 0) {
                    log.error("获取余额信息失败，accountId:{},返回信息：{}", oauthToken.getAccountId(), result);
                    return;
                }
                JSONObject data = result.getJSONObject("data");
                if (!Check.isNull(data)) {
                    BigDecimal grant = data.getBigDecimal("grant");
                    data.put("grant_amount", grant);
                    bytedanceFundMapper.replaceFundGet(data);
                    log.error("余额信息入库完成，accountId:{}", oauthToken.getAccountId());
                }
            }

        }

    }
}
