package cn.com.ctop.job.bytedance.data.service.impl;

import cn.com.ctop.job.bytedance.data.entity.Project;
import cn.com.ctop.job.bytedance.data.entity.UserAllocation;
import cn.com.ctop.job.bytedance.data.mapper.UserAllocationMapper;
import cn.com.ctop.job.bytedance.data.service.IUserAllocationService;
import cn.com.ctop.job.bytedance.data.utils.Check;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 用户分配
 */
@Service
public class UserAllocationServiceImpl extends ServiceImpl<UserAllocationMapper, UserAllocation> implements IUserAllocationService {

    @Autowired
    UserAllocationMapper userAllocationMapper;

    @Override
    public UserAllocation getByAccountId(Long accountId) {
        return userAllocationMapper.getOneByAccountId();
    }

    @Override
    public List<UserAllocation> listByMediaId(String mediaId, int accountStatus) {
        return userAllocationMapper.listByMediaId(mediaId, accountStatus);
    }

    @Override
    public String getAdvertiserIdByAccountId(Long accountId) {
        return userAllocationMapper.queryAdvertiserId(accountId);
    }

    @Override
    public List<JSONObject> getAccountListByProject(Long projectId) {
        return userAllocationMapper.getAccountListByProject(projectId);
    }

    /**
     * 获取项目下的所有userId
     *
     * @param projectId
     * @return
     */
    @Override
    public List<JSONObject> getUserIdListByProjectId(Long projectId) {
        return userAllocationMapper.getUserIdListByProjectId(projectId);
    }

    @Override
    public List<JSONObject> getAccountIdListByUserId(String userId) {
        return userAllocationMapper.getAccountIdListByUserId(userId);
    }

    @Override
    public JSONObject getSwitchStatusByAccount(Long accountId) {
        return userAllocationMapper.getSwitchStatusByAccount(accountId);
    }

    @Override
    public List<Long> getAccountsByDeleteComment() {
        return userAllocationMapper.queryAccountsByDeleteComment();
    }

    @Override
    public List<UserAllocation> getAccountListByUserIdAndProductId(String userId, Long projectId, Long productId) {
        return userAllocationMapper.getAccountListByUserIdAndProductId(userId, projectId, productId);
    }

    @Override
    public Map<Long, List<JSONObject>> getAccountListByProjectList(List<JSONObject> projectList) {
        Map<Long, List<JSONObject>> map = new HashMap<>();
        List list = new ArrayList();
        for (int i = 0; i < projectList.size(); i++) {
            JSONObject jsonObject = projectList.get(i);
            if (!Check.isNull(jsonObject)) {
                list.add(jsonObject.getLong("projectId"));
                map.put(jsonObject.getLong("projectId"), new ArrayList<>());
            }
        }
        List<JSONObject> jsonList = userAllocationMapper.getAccountListByProjectList(list);
        for (int i = 0; i < jsonList.size(); i++) {
            JSONObject jsonObject = jsonList.get(i);
            Long projectId = jsonObject.getLong("projectId");
            List<JSONObject> jsonObjects = map.get(projectId);
            jsonObjects.add(jsonObject);
        }
        return map;
    }

    @Override
    public List<JSONObject> getUserIdListByProductId(Long productId) {
        return userAllocationMapper.getUserIdListByProductId(productId);
    }

    @Override
    public List<Long> getValidProjectIds(Integer mediaId) {
        return userAllocationMapper.getValidProjectIds(mediaId);
    }

    @Override
    public List<JSONObject> getByAccountIds(JSONArray accountIds) {
        return userAllocationMapper.getByAccountIds(accountIds);
    }

    @Override
    public List<Long> getKuaiShouUserIdListByProjectId(Long projectId) {
        return userAllocationMapper.getKuaiShouUserIdListByProjectId(projectId);
    }

    @Override
    public List<JSONObject> selectAccountsByKuaishouUserId(Long userId) {
        return userAllocationMapper.selectAccountsByKuaishouUserId(userId);
    }

    @Override
    public boolean updateProjectStrategyStatus(Long accountId, Integer projectStrategy) {
        int num = userAllocationMapper.updateProjectStrategyStatus(accountId, projectStrategy);
        return num > 0;
    }

    @Override
    public List<Long> queryAutomaticAccounts(Long projectId) {
        return userAllocationMapper.queryAutomaticAccounts(projectId);
    }

    @Override
    public List<Long> getIdList() {
        return userAllocationMapper.getIdList();
    }

    @Override
    public JSONObject getProjectByAccountId(Long accountId) {
        return baseMapper.getProjectByAccountId(accountId);
    }

    @Override
    public Project getProjectEntity(Long projectId) {
        return userAllocationMapper.getProjectEntity(projectId);
    }
}
