package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.data.service.IByteDanceHourlyAccountWarningService;
import cn.com.ctop.job.bytedance.data.service.IOauthTokenService;
import cn.com.ctop.job.bytedance.data.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 头条预警
 *
 * @author yumeng
 */
@Component
public class ByteDanceHourlyAccountWarningLoadJob {
    @Autowired
    private IOauthTokenService tokenService;
    @Autowired
    private IByteDanceHourlyAccountWarningService byteDanceHourlyAccountWarningService;
    static ExecutorService executorService = Executors.newFixedThreadPool(3);

    /**
     * 头条时段消耗预警
     *
     * @throws Exception
     */
    @XxlJob("byteDanceHourlyAccountWarning")
    public void execute() throws Exception {
        XxlJobHelper.log("开始时段消耗预警定时任务");
        long startTime = System.currentTimeMillis();
        List<Long> tokens = tokenService.getAllToutiaoAccountIds();
        if (null == tokens || tokens.isEmpty()) {
            XxlJobHelper.log("定时获取头条数据异常：未获取到可用的token");
            XxlJobHelper.handleFail();
            return;
        }
        String nowDateStr = DateUtils.getNowDate("yyyy-MM-dd");
        Integer nowDateInt = DateUtils.getIntegerDate(nowDateStr);
        Integer yesterdaInt = DateUtils.getIntegerDate(DateUtils.getAnotherDay("yyyy-MM-dd", nowDateStr, -1));
        tokens.forEach(accountId -> executorService.submit(() -> byteDanceHourlyAccountWarningService.byteDanceAccountWarning(accountId, nowDateInt, yesterdaInt)));
        XxlJobHelper.log("时段消耗预警定时任务执行完毕，耗时：{} ms", System.currentTimeMillis() - startTime);
    }
}
