package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.data.constant.CtopAdConstant;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.service.IByteDanceAdvertiserDataService;
import cn.com.ctop.job.bytedance.data.service.IOauthTokenService;
import cn.com.ctop.job.bytedance.data.utils.Check;
import cn.com.ctop.job.bytedance.data.utils.DateUtils;
import cn.com.ctop.job.bytedance.utils.RedisUtil;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 广告主定时任务
 *
 * @throws Exception
 */
@Slf4j
@Component
public class BytedanceAdvertiseLoadJob {
    @Autowired
    private IOauthTokenService tokenService;

    private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService hourExecutorService = Executors.newFixedThreadPool(3);
    @Autowired
    private IByteDanceAdvertiserDataService advertiserDataService;


    @Autowired
    private RedisUtil redisUtil;

    /**
     * 获取广告主日报-遗漏数据补充
     *
     * @throws Exception
     */
    @XxlJob("bytedanceAdvertiserDailyReportSupplement")
    public void bytedanceAdvertiserDailyReportSupplement() throws Exception {

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");

        //获取广告主信息异常的key
        Set<String> set = redisUtil.kesy("ADVERTISER*");
        if (Check.isNull(set)) {
            return;
        }

        List<String> list = new ArrayList<>(set);

        for (String key : list) {

            //添加重试
            List<Object> adAccountList;
            adAccountList = redisUtil.getList(key, null);
            //遍历key下的所有信息
            for (Object accountId : adAccountList) {

                // log.info("广告主遗漏数据补充,日期:{},accountId:{}",key,adAccountList.toString());
                String token = tokenService.getByAccountId(Long.valueOf(String.valueOf(accountId)));
                if (Check.isNull(token)) {
                    log.error("此账户未获取到相关token，accountId：{}", accountId);
                    return;
                }

                OauthToken oauthToken = new OauthToken();
                oauthToken.setAccessToken(token);
                oauthToken.setAccountId(Long.valueOf(String.valueOf(accountId)));
                String time = key.replaceAll("ADVERTISER", "");
                if(Check.isNull(time)){
                    continue;
                }
                advertiserDataService.getAdvertiserReport(oauthToken, time, time, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY);

                adAccountList = redisUtil.getList(key, null);

                if (adAccountList.size() == 0) {
                    redisUtil.delete(key);
                }

                try {
                    Thread.sleep(1000 * 2);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }

            }

        }


    }


    /**
     * 广告主日报
     *
     * @throws Exception
     */
    @XxlJob("bytedanceAdvertiserDailyReport")
    public void bytedanceAdvertiserDailyReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        advertiserDataService.getAdvertiserReport(oauthToken, nowDate, nowDate, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY);
                        advertiserDataService.getAdvertiserReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY);
                        advertiserDataService.getAdvertiserReport(oauthToken, yesterda2, yesterda2, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY);
                    }
                });
    }

    /**
     * 广告主日报 分版位
     *
     * @throws Exception
     */
    @XxlJob("bytedancePositionAdvertiserDailyReport")
    public void bytedancePositionAdvertiserDailyReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
//                        advertiserDataService.getPositionAdvertiserReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY);
//                        advertiserDataService.getPositionAdvertiserReport(oauthToken, yesterda2, yesterda2, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY);
                        advertiserDataService.getPositionAdvertiserReport(oauthToken, yesterda2, nowDate, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY);
                    }
                });
    }

    /**
     * 广告主时报
     *
     * @throws Exception
     */
    @XxlJob("bytedanceAdvertiserHourlyReport")
    public void bytedanceAdvertiserHourlyReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));

        Date getDate = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH");
        String hour = simpleDateFormat.format(getDate);

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);

        hourExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        if ("00".equals(hour)) {
                            advertiserDataService.getAdvertiserReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY);
                        }
                        advertiserDataService.getAdvertiserReport(oauthToken, nowDate, nowDate, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY);
                        advertiserDataService.getAdvertiserReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY);

                    }
                });
    }


    /**
     * 广告主日报
     *
     * @throws Exception
     */
    @XxlJob("bytedanceAdvertiserV3DailyReport")
    public void bytedanceAdvertiserV3DailyReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        advertiserDataService.getV3AdvertiserReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_V3_TYPE_DAILY);
                        advertiserDataService.getV3AdvertiserReport(oauthToken, yesterda2, yesterda2, CtopAdConstant.BYTEDANCE_REPORT_V3_TYPE_DAILY);
                    }
                });
    }

    /**
     * 广告日报
     *
     * @throws Exception
     */
    @XxlJob("bytedanceV3PromotionProjectReport")
    public void bytedanceV3PromotionProjectReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        advertiserDataService.getV3PromotionProjectReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_V3_TYPE_DAILY);
                        advertiserDataService.getV3PromotionProjectReport(oauthToken, yesterda2, yesterda2, CtopAdConstant.BYTEDANCE_REPORT_V3_TYPE_DAILY);
                    }
                });
    }

    /**
     * 广告时报
     *
     * @throws Exception
     */
    @XxlJob("bytedanceV3PromotionProjectReportHour")
    public void bytedanceV3PromotionProjectReportHour() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
//        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        advertiserDataService.getV3PromotionProjectReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_V3_TYPE_HOURLY);
                        advertiserDataService.getV3PromotionProjectReport(oauthToken, nowDate, nowDate, CtopAdConstant.BYTEDANCE_REPORT_V3_TYPE_HOURLY);
                    }
                });
    }

    /**
     * 广告主时报
     *
     * @throws Exception
     */
    @XxlJob("bytedanceAdvertiserV3HourlyReport")
    public void bytedanceAdvertiserV3HourlyReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));

        Date getDate = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH");
        String hour = simpleDateFormat.format(getDate);

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);

        hourExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        if ("00".equals(hour)) {
                            advertiserDataService.getV3AdvertiserReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_V3_TYPE_HOURLY);
                        }
                        advertiserDataService.getV3AdvertiserReport(oauthToken, nowDate, nowDate, CtopAdConstant.BYTEDANCE_REPORT_V3_TYPE_HOURLY);
                        advertiserDataService.getV3AdvertiserReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_V3_TYPE_HOURLY);

                    }
                });
    }

    /**
     * master
     */
    @XxlJob("bytedanceAdvertiserHourlyReport2")
    public void bytedanceAdvertiserHourlyReport2() {
        String token = tokenService.getByAccountId(1714136819636232L);
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(1714136819636232L);
        log.info("测试测试===");
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        advertiserDataService.getAdvertiserReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY);
    }


}
