package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.service.IBytedanceAudiencePackageGetService;
import cn.com.ctop.job.bytedance.data.service.IOauthTokenService;
import cn.com.ctop.job.bytedance.data.utils.Check;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 广告计划定时任务
 *
 * @throws Exception
 */
@Slf4j
@Component
public class BytedanceAudiencePackageLoadJob {
    @Autowired
    private IOauthTokenService tokenService;

    private static ExecutorService hourExecutorService = Executors.newFixedThreadPool(3);
    @Autowired
    private IBytedanceAudiencePackageGetService audiencePackageGetService;

    /**
     * 获取定向包数据
     *
     * @throws Exception
     */
    @XxlJob("bytedanceAudiencePackageGetLoadJob")
    public void bytedanceAudiencePackageGetLoadJob() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }

        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        hourExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        audiencePackageGetService.getAudiencePackageGetReport(oauthToken);
                    }
                });

    }


}
