package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.data.constant.CtopAdConstant;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.service.IBytedanceCampaignGetService;
import cn.com.ctop.job.bytedance.data.service.IBytedanceCampaignReportDailyService;
import cn.com.ctop.job.bytedance.data.service.IBytedanceCampaignReportHourlyService;
import cn.com.ctop.job.bytedance.data.service.IOauthTokenService;
import cn.com.ctop.job.bytedance.data.utils.Check;
import cn.com.ctop.job.bytedance.data.utils.DateUtils;
import cn.com.ctop.job.bytedance.utils.RedisUtil;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 广告组定时任务
 *
 * @throws Exception
 */
@Slf4j
@Component
public class BytedanceCampaignLoadJob {
    @Autowired
    private IOauthTokenService tokenService;

    private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService hourExecutorService = Executors.newFixedThreadPool(3);
    @Autowired
    private IBytedanceCampaignGetService campaignDataService;
    @Autowired
    IBytedanceCampaignReportDailyService campaignReportDailyService;
    @Autowired
    IBytedanceCampaignReportHourlyService campaignReportHourlyService;


    @Autowired
    private RedisUtil redisUtil;


    /**
     * 广告组遗漏数据补充
     * @throws Exception
     */
    @XxlJob("bytedanceCampaignLoadJobSupplement")
    public void bytedanceCampaignLoadJobSupplement() throws Exception {
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        //String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);

       Set<String> set = redisUtil.kesy("CAMP*");
       if (Check.isNull(set)){
           return;
       }

       List<String> list = new ArrayList<>(set);

        for (String yesterda : list) {

            //添加重试
            List<Object> campAccountList;
            campAccountList = redisUtil.getList(yesterda,null);
            for (Object accountId : campAccountList) {
                //log.info("广告组遗漏数据补充,日期:{},accountId:{}",yesterda,campAccountList.toString());
                String token = tokenService.getByAccountId(Long.valueOf(String.valueOf(accountId)));
                if (Check.isNull(token)) {
                    log.error("此账户未获取到相关token，accountId：{}", accountId);
                    return;
                }

                OauthToken oauthToken = new OauthToken();
                oauthToken.setAccessToken(token);
                oauthToken.setAccountId(Long.valueOf(String.valueOf(accountId)));
                campaignDataService.getCampaignData(oauthToken, yesterda.replaceAll("CAMP",""), yesterda.replaceAll("CAMP",""), 1, 1000);
                campAccountList = redisUtil.getList(yesterda,null);
                if (campAccountList.size() == 0){
                    redisUtil.delete(yesterda);
                }

                try {
                    Thread.sleep(1000*3);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }

            }

        }





    }



    /**
     * 获取获取广告组信息
     *
     * @throws Exception
     */
    @XxlJob("bytedanceCampaignLoadJob")
    public void bytedanceCampaignLoadJob() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }


            String token = tokenService.getByAccountId(Long.valueOf(param));
            if (Check.isNull(token)) {
                log.error("此账户未获取到相关token，accountId：{}", param);
                return;
            }
            String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
            String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
            String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
            String yesterda3 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -3);
            OauthToken oauthToken = new OauthToken();
            oauthToken.setAccessToken(token);
            oauthToken.setAccountId(Long.parseLong(param));

            dailyExecutorService.submit(
                    new Runnable() {
                        @Override
                        public void run() {
                            campaignDataService.getCampaignData(oauthToken, yesterda, yesterda, 1, 1000);
                            campaignDataService.getCampaignData(oauthToken, yesterda2, yesterda2, 1, 1000);
                            campaignDataService.getCampaignData(oauthToken, yesterda3, yesterda3, 1, 1000);
                        }
                    });


}

    /**
     * 广告组日报
     *
     * @throws Exception
     */
    @XxlJob("bytedanceCampaignDailyReport")
    public void accountNowDateHourReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        campaignReportDailyService.getCampaignDailyReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY, 1, 1000);
                        campaignReportDailyService.getCampaignDailyReport(oauthToken, yesterda2, yesterda2, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY, 1, 1000);
                    }
                });
    }

    /**
     * 广告组时报
     *
     * @throws Exception
     */
    @XxlJob("bytedanceCampaignHourlyReport")
    public void accountYesterdayHourReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));

        Date getDate = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH");
        String hour = simpleDateFormat.format(getDate);

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        hourExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        if ("00".equals(hour)) {
                            campaignReportHourlyService.getCampaignHourlyReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY, 1, 1000);
                        }
                        campaignReportHourlyService.getCampaignHourlyReport(oauthToken, nowDate, nowDate, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY, 1, 1000);
                        campaignReportHourlyService.getCampaignHourlyReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY, 1, 1000);

                    }
                });
    }
}
