package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.data.constant.CtopAdConstant;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.service.IBytedanceCreativeGetService;
import cn.com.ctop.job.bytedance.data.service.IBytedanceCreativeReportDailyService;
import cn.com.ctop.job.bytedance.data.service.IBytedanceCreativeReportHourlyService;
import cn.com.ctop.job.bytedance.data.service.IOauthTokenService;
import cn.com.ctop.job.bytedance.data.utils.Check;
import cn.com.ctop.job.bytedance.data.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 广告组定时任务
 *
 * @throws Exception
 */
@Slf4j
@Component
public class BytedanceCreativeLoadJob {
    @Autowired
    private IOauthTokenService tokenService;

    private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService hourExecutorService = Executors.newFixedThreadPool(3);
    @Autowired
    private IBytedanceCreativeGetService creativeGetService;
    @Autowired
    IBytedanceCreativeReportDailyService creativeReportDailyService;
    @Autowired
    IBytedanceCreativeReportHourlyService creativeReportHourlyService;

    /**
     * 获取广告创意列表信息
     *
     * @throws Exception
     */
    @XxlJob("bytedanceCreativeLoadJob")
    public void bytedanceCreativeLoadJob() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }

        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        creativeGetService.getCreativeData(oauthToken, yesterda, yesterda, 1, 1000);
        creativeGetService.getCreativeData(oauthToken, yesterda2, yesterda2, 1, 1000);
    }

    /**
     * 广告创意日报
     *
     * @throws Exception
     */
    @XxlJob("bytedanceCreativeDailyReport")
    public void accountNowDateHourReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        creativeReportDailyService.getCreativeDailyReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY, 1, 1000);
                        creativeReportDailyService.getCreativeDailyReport(oauthToken, yesterda2, yesterda2, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY, 1, 1000);
                    }
                });
    }

    /**
     * 广告创意时报
     *
     * @throws Exception
     */
    @XxlJob("bytedanceCreativeHourlyReport")
    public void accountYesterdayHourReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));

        Date getDate = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH");
        String hour = simpleDateFormat.format(getDate);

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        hourExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        if ("00".equals(hour)) {
                            creativeReportHourlyService.getCampaignHourlyReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY, 1, 1000);
                        }
                        creativeReportHourlyService.getCampaignHourlyReport(oauthToken, nowDate, nowDate, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY, 1, 1000);
                        creativeReportHourlyService.getCampaignHourlyReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY, 1, 1000);

                    }
                });
    }
}
