package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.service.IBytedanceFileImageGetService;
import cn.com.ctop.job.bytedance.data.service.IBytedanceFileVideoGetService;
import cn.com.ctop.job.bytedance.data.service.IOauthTokenService;
import cn.com.ctop.job.bytedance.data.utils.Check;
import cn.com.ctop.job.bytedance.data.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 头条素材定时任务
 *
 * @throws Exception
 */
@Slf4j
@Component
public class BytedanceFileGetLoadJob {
    @Autowired
    private IOauthTokenService tokenService;

    private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService hourExecutorService = Executors.newFixedThreadPool(3);
    @Autowired
    private IBytedanceFileImageGetService fileImageService;

    @Autowired
    private IBytedanceFileVideoGetService fileVideoService;

    /**
     * 获取头条图片素材
     *
     * @throws Exception
     */
    @XxlJob("bytedanceFileImageGet")
    public void bytedanceFileImageGet() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        fileImageService.getFileImageData(oauthToken, yesterda, yesterda, 1, 100);
        fileImageService.getFileImageData(oauthToken, yesterda2, yesterda2, 1, 100);
    }


    /**
     * 获取头条视频素材
     *
     * @throws Exception
     */
    @XxlJob("bytedanceFileVideoGet")
    public void bytedanceFileVideoGet() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        fileVideoService.getFileVideoData(oauthToken, yesterda, yesterda, 1, 100);
        fileVideoService.getFileVideoData(oauthToken, yesterda2, yesterda2, 1, 100);
    }



    /**
     * 获取头条视频素材 全部
     *
     * @throws Exception
     */
    @XxlJob("bytedanceFileVideoGetAll")
    public void bytedanceFileVideoGetAll() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        log.info("获取头条视频全量素材---账户--{}======》》》",oauthToken.getAccountId());
        fileVideoService.getFileVideoData(oauthToken, null, null, 1, 100);
    }



}
