package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.data.constant.CtopAdConstant;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.service.IByteDanceAdvertiserDataService;
import cn.com.ctop.job.bytedance.data.service.IOauthTokenService;
import cn.com.ctop.job.bytedance.data.utils.Check;
import cn.com.ctop.job.bytedance.data.utils.DateUtils;
import cn.com.ctop.job.bytedance.data.utils.HttpUtils;
import cn.com.ctop.job.bytedance.utils.RedisUtil;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 广告计划定时任务
 *
 * @throws Exception
 */
@Slf4j
@Component
public class BytedancePlanLoadJob {
    @Autowired
    private IOauthTokenService tokenService;

    private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService hourExecutorService = Executors.newFixedThreadPool(3);
    @Autowired
    private IByteDanceAdvertiserDataService advertiserDataService;

    @Autowired
    private RedisUtil redisUtil;

    /**
     * 获取广告计划列表信息-遗漏数据补充
     *
     * @throws Exception
     */
    @XxlJob("bytedancePlanLoadJobSupplement")
    public void bytedancePlanLoadJobSupplement() throws Exception {

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        //String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);

        Set<String> set = redisUtil.kesy("AD*");
        if (Check.isNull(set)) {
            return;
        }

        List<String> list = new ArrayList<>(set);
        for (String yesterda : list) {

            //添加重试
            List<Object> adAccountList;
            adAccountList = redisUtil.getList(yesterda, null);
            for (Object accountId : adAccountList) {

                // log.info("广告计划遗漏数据补充,日期:{},accountId:{}",yesterda,adAccountList.toString());
                String token = tokenService.getByAccountId(Long.valueOf(String.valueOf(accountId)));
                if (Check.isNull(token)) {
                    log.error("此账户未获取到相关token，accountId：{}", accountId);
                    return;
                }

                OauthToken oauthToken = new OauthToken();
                oauthToken.setAccessToken(token);
                oauthToken.setAccountId(Long.valueOf(String.valueOf(accountId)));
                advertiserDataService.getAdvertiserPlan(oauthToken, "", null, yesterda.replaceAll("AD", ""));

                adAccountList = redisUtil.getList(yesterda, null);

                if (adAccountList.size() == 0) {
                    redisUtil.delete(yesterda);
                }

                try {
                    Thread.sleep(1000 * 2);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }

            }

        }


    }


    /**
     * 获取广告计划列表信息
     *
     * @throws Exception
     */
    @XxlJob("bytedancePlanLoadJob")
    public void bytedancePlanLoadJob() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }

        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        String yesterda3 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -3);
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));

        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        advertiserDataService.getAdvertiserPlan(oauthToken, "", null, yesterda);
                        advertiserDataService.getAdvertiserPlan(oauthToken, "", null, yesterda2);
                        advertiserDataService.getAdvertiserPlan(oauthToken, "", null, yesterda3);
                    }
                });
    }

    /**
     * 计划日报
     *
     * @throws Exception
     */
    @XxlJob("bytedancePlanDailyReport")
    public void accountNowDateHourReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        advertiserDataService.getAdvertiserPlanReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY);
                        advertiserDataService.getAdvertiserPlanReport(oauthToken, yesterda2, yesterda2, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY);
                    }
                });
    }

    /**
     * 计划时报
     *
     * @throws Exception
     */
    @XxlJob("bytedancePlanHourlyReport")
    public void accountYesterdayHourReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));

        Date getDate = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH");
        String hour = simpleDateFormat.format(getDate);

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);

        hourExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        if ("00".equals(hour)) {
                            advertiserDataService.getAdvertiserPlanReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY);
                        }
                        advertiserDataService.getAdvertiserPlanReport(oauthToken, nowDate, nowDate, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY);
                        advertiserDataService.getAdvertiserPlanReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY);

                    }
                });
    }

    /**
     * 第三方监测链接预警
     *
     * @throws Exception
     */
    @XxlJob("bytedanceActionTrackUrlMonitorJob")
    public void bytedanceActionTrackUrlMonitorJob() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
//       String urlPrefix = "http://192.168.1.43:8808/bytedance-api/ad/bytedancePlanJobController/monitorBytedanceActionTrackUrl";
//       String urlPrefix = "http://139.186.165.84:8808/bytedance-api/ad/bytedancePlanJobController/monitorBytedanceActionTrackUrl";
        String urlPrefix = "http://118.24.244.213:8808/bytedance-api/ad/bytedancePlanJobController/monitorBytedanceActionTrackUrl";

        Map<String, Object> map = new HashMap();
        map.put("accountId", param);
        HttpUtils.httpGet(urlPrefix, map, null);
    }
}
