package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.service.IBytedanceProjectService;
import cn.com.ctop.job.bytedance.data.service.IOauthTokenService;
import cn.com.ctop.job.bytedance.data.utils.Check;
import cn.com.ctop.job.bytedance.data.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 获取项目列表
 */
@Slf4j
@Component
public class BytedanceProjectJob {
    @Autowired
    private IOauthTokenService tokenService;

    private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService ocpxActionTypeExecutorService = Executors.newFixedThreadPool(5);


    @Autowired
    private IBytedanceProjectService projectService;


    /**
     * 获取项目列表
     */
    @XxlJob("getV3Project")
    public void getV3Project() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        projectService.getProjectList(oauthToken, nowDate, nowDate, 1);
                        projectService.getProjectList(oauthToken, yesterda, yesterda, 1);
                        projectService.getProjectList(oauthToken, yesterda2, yesterda2, 1);
                    }
                });


    }

    /**
     * 计划转化目标、转化出价预警
     *
     * @throws Exception
     */
    @XxlJob("ocpxActionTypeTrackUrlJob")
    public void ocpxActionTypeTrackUrlJob() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
//        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
//        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        ocpxActionTypeExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        projectService.checkAdvertiserProject(oauthToken, nowDate, 1);
                    }
                });
    }

}
