package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.service.IBytedancePromotionListService;
import cn.com.ctop.job.bytedance.data.service.IOauthTokenService;
import cn.com.ctop.job.bytedance.data.utils.Check;
import cn.com.ctop.job.bytedance.data.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 获取体验版广告列表
 *
 * @author zianY
 * 2022-10-19
 */
@Slf4j
@Component
public class BytedancePromotionJob {
    @Autowired
    private IOauthTokenService tokenService;

    private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);


    @Autowired
    private IBytedancePromotionListService promotionListService;


    /**
     * 体验版-获取广告列表
     *
     * @throws Exception
     */
    @XxlJob("getPromotionList")
    public void getPromotionList() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        String yesterda3 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -3);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        promotionListService.getPromotionList(oauthToken, yesterda, yesterda, 1,10);
                        promotionListService.getPromotionList(oauthToken, yesterda2, yesterda2, 1,10);
                        promotionListService.getPromotionList(oauthToken, yesterda3, yesterda3, 1,10);
                    }
                });


    }

}
